#include "chess.h"

#define MIN(X, Y)  ((X) < (Y) ? (X) : (Y))

//Zeit-Modi definieren
#define MODI_COUNT 20

//Eeingabe Modi definieren
#define MODE_HOUR_MIN 0
#define MODE_SEC 1
#define MODE_GUILLOTINE1 2
#define MODE_GUILLOTINE2 3
#define MODE_GUILLOTINE3 4
#define MODE_GUILLOTINE4 5
#define MODE_GUILLOTINE5 6
#define MODE_GUILLOTINE6 7
#define MODE_GUILLOTINE7 8
#define MODE_GUILLOTINE_COUNT 9
#define MODE_BONUS 10
#define MODE_MOVES_COUNT1 11
#define MODE_MOVES_COUNT2 12
#define MODE_MOVES_COUNT3 13
#define MODE_MOVES_COUNT4 14
#define MODE_MOVES_COUNT5 15
#define MODE_MOVES_COUNT6 16
#define MODE_MOVES_COUNT7 17
#define MODE_MOVES 18

#define ABORT 0
#define FINISHED 1

//Max Werte definieren

#define MAX_TIME 359999        //32bit Wert, aber das Display kann nur 99.99 anzeigen also 100 Stunden - 1 Sekunde
#define MAX_BONUS 5999         //100 Minuten - 1 Sekunde
#define MAX_GUILLOTINE 65535   //16bit Wert ^= 18 Stunden, 12 Minuten (und 15 Sekunden); 2^16 - 1 = 65535
#define MAX_MOVES 9999         //16bit Wert, aber nur 9999 als Anzeigemglichkeit


static Game Games[MODI_COUNT] PROGMEM = { 
//Seconds,bonus,options,{guillotine1,2,...}
    {300, 0, 0, {0}},                                                                                       //01  	Blitz  	5 min
    {1500, 0, 0, {0}},                                                                                      //02  	Schnellschach  	25 min
    {0,0,GAME_OPTION_MANUAL,{0}},                                                                           //03  	Blitz / Schnellschach  	manuelle Einstellung
    {7200,0,GAME_OPTION_GUILLOTINE1,{1800}},                                                                //04  	Turnier + Guillotine  	2 Std. + 30 min. / Rest
    {0,0,GAME_OPTION_MANUAL | GAME_OPTION_GUILLOTINE1,{0}},                                                 //05  	Turnier + Guillotine  	manuelle Einstellung
    {7200,0,GAME_OPTION_GUILLOTINE2,{3600,1800}},                                                           //06  	2 x Turnier + Guillotine  	2 Std. + 1 Std. + 30 min. / Rest
    {0,0,GAME_OPTION_MANUAL | GAME_OPTION_GUILLOTINE2,{0,0}},                                               //07  	2 x Turnier + Guillotine  	manuelle Einstellung
    {7200,0,GAME_OPTION_WDH_GUILLOTINE|GAME_OPTION_GUILLOTINE1,{0}},                                        //08  	Turnier + wiederholende 2. Periode  	2 Std. + 1 Std. (Wdh.)
    {0,0,GAME_OPTION_MANUAL | GAME_OPTION_WDH_GUILLOTINE|GAME_OPTION_GUILLOTINE1,{0}},                      //09  	Turnier + wiederholende 2. Periode  	manuelle Einstellung
    {1500, 10, GAME_OPTION_GUILLOTINE1 | GAME_OPTION_LATE_BONUS | GAME_OPTION_BONUS, {300}},                //10  	Turnier + "Fischer"-Periode  	25 min + (5 min + 10 Sek./Zug)
    {7200, 30, GAME_OPTION_GUILLOTINE1 | GAME_OPTION_LATE_BONUS | GAME_OPTION_BONUS, {900}},                //11  	Turnier + "Fischer"-Periode  	2 Std. + (15 min + 30 Sek./Zug)
    {0,0,GAME_OPTION_MANUAL | GAME_OPTION_GUILLOTINE1 | GAME_OPTION_LATE_BONUS | GAME_OPTION_BONUS, {0}},   //12  	Turnier + "Fischer"-Periode  	manuelle Einstellung
    {7200,30,GAME_OPTION_GUILLOTINE2 | GAME_OPTION_LATE_BONUS | GAME_OPTION_BONUS, {3600,900}},             //13  	2 x Turnier + "Fischer"-Periode  	2 Std. + 1 Std. + (15 min + 30 Sek./Zug) 
    {0,0, GAME_OPTION_MANUAL | GAME_OPTION_GUILLOTINE2 | GAME_OPTION_LATE_BONUS | GAME_OPTION_BONUS, {0,0}},//14  	2 x Turnier + "Fischer"-Periode  	manuelle Einstellung 
    {180,2,GAME_OPTION_BONUS,{0}},                                                                          //15  	Bonus ("Fischer") Blitz  	3 min + 2 Sek./Zug
    {1500,10,GAME_OPTION_BONUS,{0}},                                                                        //16  	Bonus ("Fischer") Schnellschach  	25 min + 10 Sek./Zug
    {5400,30,GAME_OPTION_BONUS,{0}},                                                                        //17  	Bonus ("Fischer") Langsam  	90 min + 30 Sek./Zug
    {0,0,GAME_OPTION_MANUAL | GAME_OPTION_BONUS ,{0}},                                                      //18  	Bonus ("Fischer")  	manuelle Einstellung
    {5400,30,GAME_OPTION_BONUS | GAME_OPTION_GUILLOTINE1  | GAME_OPTION_USE_MOVES, {900},{40}},                          //19  	Bonus Turnier  	90 min + 15 min (alle Zge + 30 sek./Zug)
    {6000,30,GAME_OPTION_BONUS | GAME_OPTION_GUILLOTINE1 | GAME_OPTION_USE_MOVES, {1800},{40}},                          //20  	Bonus Turnier  	100 min + 30 min (alle Zge + 30 sek./Zug)
    {0,0,GAME_OPTION_MANUAL | GAME_OPTION_FREE_GUILLOTINE | GAME_OPTION_BONUS  | GAME_OPTION_USE_MOVES, {0},{0}},    //21  	Bonus Turnier (bis zu 4 Perioden)  	manuelle Einstellung (mit oder ohne Zugzhler)
    {300,3,GAME_OPTION_BONUS | GAME_OPTION_FREE_BONUS, {0}},                                                //22  	"Bronstein"-Modus  	5 min + 3 sek./Zug (frei)
    {1500,10,GAME_OPTION_BONUS | GAME_OPTION_FREE_BONUS,{0}},                                               //23  	"Bronstein"-Modus  	25 min + 10 sek./Zug (frei)
    {6900,5,GAME_OPTION_BONUS | GAME_OPTION_FREE_BONUS,{0}},                                                //24  	"Bronstein"-Modus  	1 Std. 55 min + 5 sek./Zug (frei)
    {0,0,GAME_OPTION_BONUS | GAME_OPTION_FREE_BONUS,{0}},                                                   //25  	"Bronstein"-Modus  	manuelle Einstellung
    {3600,20,GAME_OPTION_BYOYOMI | GAME_OPTION_GUILLOTINE1 | GAME_OPTION_BONUS,{0}},                        //26  	Go mit byo-yomi  	1 Std. + 1x20 sek. byo-yomi
    {7200,30,GAME_OPTION_BYOYOMI | GAME_OPTION_GUILLOTINE1,{0}},                                            //27  	Go mit byo-yomi  	2 Std. + 1x30 sek. byo-yomi
    {0,0,GAME_OPTION_MANUAL | GAME_OPTION_BYOYOMI | GAME_OPTION_GUILLOTINE1 | GAME_OPTION_BONUS, {0}}       //28  	Go mit byo-yomi  	manuelle Einstellung
    /* TODO (#1#): MODE 19-21 Zge zum umprung eintragen */
    
    };
    
volatile Game currentGame[2];



//Zhlvariablen initialisieren       
volatile uint8_t hours[] = {0,0};
volatile uint8_t seconds[] = {0,0};
volatile uint8_t minutes[] = {0,0};
volatile uint8_t mseconds[] = {0,0};
volatile uint8_t updateDisplay = 0;
volatile uint8_t seconds10 = 0; 
volatile uint8_t guillotine = 0;

unsigned long Timer = 0;
unsigned long Timer2 = 0;

//Andere Variablen
uint8_t Intensity = 20;

//Schach Variablen
uint8_t current = 0;
uint8_t pause = 1;
uint8_t game_mode = 1;
volatile int8_t lost = -1;
uint16_t moves = 0;
int8_t beginner = -1;

Game create_game(uint16_t time, uint16_t bonus)
{
    Game temp;
    temp.time = time;
    temp.bonus = bonus;
    return temp;   
}

void ChessInit(void)
{
    //Init MAX7219
    SPIMasterInit();
    SetMode(MAX_MODE_NORMAL);
    SetIntensity(Intensity / 10);
    SetDecodeMode(0x00);
    SetScanLimit(0x07);
    SetAll(0);
    
    //Inputs
    DDRD &= ~((1 << 6) | (1 << 7));
    //Enalbe Pull-Up-Resistor
    PORTD |= (1 << 6) | (1 << 7);
        
    //Init ADC
    ADCInit();
    
    //8bit Timer Init
    TCCR0 = (1<<CS00);          //Kein Teiler
    TIMSK |= (1<<TOIE0);        // Overflow Interrupt erlauben
    sei();                      // Global Interrupts aktivieren
}

//Symbole fr wei und schwarz
uint8_t symbols[] = {8, 29};

void PlayManual(void)
{
    #define KEY_DOWN 1
    #define KEY_REPEAT 2
    #define KEY_UP 0
    #define SHOW_TIME 0
    #define SHOW_MOVES 1
    uint8_t show = SHOW_TIME;
    uint8_t playerkey = 0;      //Variablen fr die Tastenentprellung
    uint8_t playerkey1 = 0;
    uint8_t playerkey2 = 0;
    uint8_t count = 0;
    uint32_t oldtime[2];  //Fr den Bronsteinmodus mssen wir die Startzeit speichern!
    for(uint8_t i = 0; i < 2; i++)
        oldtime[i] = currentGame[i].time; //Werte der oldTime auf die aktuelle Zeit setzen!
    guillotine = 0;             //Guillotinen auf 0 setzen
    lost = -1;                  //noch hat niemand verloren
    WaitForKeyUp();             //Warten bis die Besttigungstaste losgelassen wurde, schlielich soll die Pause nicht gleich deaktiviert werden
    SetTime();                  //Die Zeit in das SecMinHour Format setzen
    //KeyTest();
    updateDisplay = 1;          //Damit auch was angezeigt wird, bevor etwas gemacht wird
    Timer = 0;                  //Wir setzen auch noch den Timer zurck
    DoBonus(0);                 //Und vergeben den ersten Bonus
    DoBonus(1);                 //Natrlich an beide
    while(1) 
    {    
        uint8_t key = GetKey();
        //FOR DEBUGING
        //if(key != 255)
        //    SetDigit(0,chars[key]);
        switch(key)
        {
            case 0:
                show = SHOW_TIME;
                break;
            case 2:
                if(pause)
                {
                    for(uint8_t i = 0; i < 2; i++)
                        currentGame[i].time = GetTime(i);
                    SetManual();
                    KeyRo(0b11110000, MODE_MOVES);
                    updateDisplay = 1;
                }
                break;
            case 4:
                if(Intensity < 0x0F * 10)
                { 
                    Intensity++;   
                    SetIntensity(Intensity / 10);
                }
                break;
            case 5:
                if(pause == 1)
                    pause = 2;
                if(pause == 0)
                    pause = 3;
                break;
            case 6:
                if(Intensity > 0x00)
                { 
                    Intensity--;   
                    SetIntensity(Intensity / 10);
                }
                break;
            case 8:
                show = SHOW_MOVES;
                break;
            case 9:
                if(pause)
                    return;
                break;
        };
        if(pause == 2 && key == 0)
        {
            pause = 0;
            if(beginner == -1)
                beginner = current;
        }
        if(pause == 3 && key == 0)
            pause = 1;
        if(PIND == playerkey)
            count++;
        else
        {
            playerkey = PIND;
            count = 0;   
        }
        if(count >= 25 && pause)
        {
            count = 0;
                if(playerkey & (1 << 6))
                {
                    if(playerkey1 == KEY_DOWN && current == 1)
                    {
                        //Dieser Code wird nur augefhrt wenn der Spieler die Uhr drckt, wenn er dran ist!
                        DoBonusFree(current, oldtime[current]);
                        DoMoves(current);
                        updateDisplay = 1;
                        current = 0;
                        oldtime[current] = GetTime(current);
                        playerkey1 = KEY_REPEAT;
                    }
                    if(playerkey1 == KEY_UP)
                        playerkey1 = KEY_DOWN;
                }
                else
                    playerkey1 = KEY_UP;
                if(playerkey & (1 << 7))
                {
                    if(playerkey2 == KEY_DOWN && current == 0)
                    {
                        DoBonusFree(current, oldtime[current]);
                        DoMoves(current);
                        updateDisplay = 1;
                        current = 1;
                        oldtime[current] = GetTime(current);
                        playerkey2 = KEY_REPEAT;
                    }
                    if(playerkey2 == KEY_UP)
                        playerkey2 = KEY_DOWN;
                }
                else
                    playerkey2 = KEY_UP;
        }
        if(show == SHOW_TIME)
        {
            if(updateDisplay == 1 ) 
            {
                // die aktuelle Uhrzeit holen. Dabei aber dafr sorgen
                // dass sich die nicht schnell mal ndern kann
                cli();
                updateDisplay = 0;

                // ab jetzt haben wir alle Zeit der Welt die Zeit auf
                // die Anzeige zu bringen
                for(uint8_t i = 0; i != 2; i++)
                {
                    if(lost == i)
                    {
                        if(seconds10 < 5)
                        {
                            SetDigit(1+4*i,chars[0]+128);
                            Show2Digits(0,2+4*i);
                            SetDigit(4*i,1);
                        }
                        else
                            for(int o = 0; o != 4; o++)
                                SetDigit(o+4*i,0);
                    }
                    else
                    {
                        if(hours[i] > 0 )
                        {
                            Show2Digits(minutes[i],2 + 4*i);
                            Show2DigitsDots(hours[i],0 + 4*i, (mseconds[i] < 5 || current != i) && (!pause || current != i) ? 2 : 0);  
                        }
                        else
                        {
                            Show2Digits(seconds[i],2 + 4*i);
                            Show2DigitsDots(minutes[i],0 + 4*i, (mseconds[i] < 5 || current != i) && (!pause || current != i) ? 2 : 0);
                        }
                    } 
                }
                sei();
            }
        }
        if(show == SHOW_MOVES)
        {
            ShowMoves();
            SetDigit(5,0);
            SetDigit(6,0);
            SetDigit(4,symbols[beginner]);
            SetDigit(7,symbols[1-beginner]);    
        }
    }
}

void ShowMoves(void)
{
    Show2Digits(moves / 100,0);
    Show2Digits(moves % 100,2);
}

void DoMoves(uint8_t player)
{
    if(!pause)
    {
        if(player != beginner)
            moves++;
        if(!( ( (currentGame[current].options >> 5) & 7) > guillotine + 1))
        {
            uint16_t allMoves = 0;
            for(uint8_t i = 0; i<guillotine + 1; i++)
                allMoves += currentGame[current].moves[guillotine];
            if(moves >= allMoves)
                DoGuillotine();
                
        }
    }   
}

void DoBonus(uint8_t player)
{
    if(currentGame[player].options & GAME_OPTION_BYOYOMI)
    {
        if(guillotine > 0)
            SetTimeForPlayer(player, currentGame[player].bonus);
    }
    else
        if(currentGame[player].options & GAME_OPTION_BONUS) //Ist Bonus berhaupt aktiviert?
            if((!(currentGame[player].options & GAME_OPTION_LATE_BONUS)) || ((currentGame[current].options >> 5) & 7) == guillotine) //Ist LateBonus aktiviert, wenn ja ist es berhaupt die letzte guillotine
                AddTime(player, currentGame[player].bonus); //Bonus hinzufgen
}

void DoBonusFree(uint8_t player, uint32_t oldTime)
{
    if(currentGame[player].options & GAME_OPTION_BYOYOMI)
    {
        if(guillotine > 0)
            SetTimeForPlayer(player, currentGame[player].bonus);
    }
    else
    if(currentGame[player].options & GAME_OPTION_BONUS) //Ist Bonus berhaupt aktiviert?
        if((!(currentGame[player].options & GAME_OPTION_LATE_BONUS)) || ((currentGame[current].options >> 5) & 7) == guillotine) //Ist LateBonus aktiviert, wenn ja ist es berhaupt die letzte guillotine
        {
            if(currentGame[player].options & GAME_OPTION_FREE_BONUS)
                AddTime(player, MIN(currentGame[player].bonus,oldTime - GetTime(player))); //Bonus hinzufgen
            else
                AddTime(player,currentGame[player].bonus);
        }
}

void AddTime(uint8_t player, uint16_t time)
{
    SetTimeForPlayer(player, GetTime(player) + time);
}

uint32_t GetTime(uint8_t player)
{
    return hours[player] * 3600 + minutes[player] * 60 + seconds[player];
}

void SetTime()
{
    for(uint8_t i = 0; i!=2; i++)
    {
        hours[i] = currentGame[i].time / 3600;
        minutes[i] = currentGame[i].time / 60 - hours[i] * 60;
        seconds[i] = currentGame[i].time - minutes[i] * 60 - hours[i] * 3600;   
    }
}

void SetTimeForPlayer(uint8_t player, uint32_t time)
{
    cli();
    hours[player] = time / 3600;
    minutes[player] = time / 60 - hours[player] * 60;
    seconds[player] = time - minutes[player] * 60 - hours[player] * 3600;
    sei();   
}

ISR (TIMER0_OVF_vect)
{
    if(!pause)
    {
        Timer += 256;
        if (Timer >= F_CPU/* / 10*/)
        {
            Timer -= F_CPU/* / 10*/;
            //mseconds[current]--; 
            updateDisplay = 1;
            if(mseconds[current] == 255)
            {
                mseconds[current] = 9;
                seconds[current]--;                      
                if( seconds[current] == 255 ) 
                {
                    seconds[current] = 59;
                    minutes[current]--;
                    if( minutes[current] == 255 ) 
                    {
                        minutes[current] = 59;
                        hours[current]--;
                        if(hours[current] == 255)
                        {
                            hours[current] = minutes[current] = seconds[current] = mseconds[current] = 0;
                            if(((currentGame[current].options >> 5) & 7) > guillotine && currentGame[current].guillotine[guillotine] > 0)
                                DoGuillotine();    
                            else
                                if(lost == -1)
                                    lost = current;
                        }
                    }
                }
            }
            updateDisplay = 1;
        }
    }
    Timer2 += 256;
    if (Timer2 >= F_CPU / 10)
    {
        Timer2 -= F_CPU / 10; 
        if(seconds10++ == 10)
            seconds10 = 0;
    }
}

void DoGuillotine(void)
{
    for(uint8_t i = 0; i < 2; i++)
    {
        AddTime(i,currentGame[i].guillotine[guillotine]);
        if(currentGame[current].options & GAME_OPTION_BYOYOMI) 
        {
            DoBonus(i);
        }
    }
    if(!(( ( (currentGame[current].options >> 5) & 7) > guillotine + 1) && (currentGame[current].options & GAME_OPTION_WDH_GUILLOTINE)))
        guillotine++;   
}

void Lost(uint8_t player)
{
    uint8_t screen = 1;
    while(1)
    {
        for(int i = 0; i != 4; i++)
            SetDigit(i + player*4, screen);
        screen = 1 - screen;
        _delay_ms(500);
    }
}

void MainMenu(void)
{
    SetAll(0);
    uint8_t key = 255;
    while(1)
    {
        Show2Digits(game_mode,0);
        key = GetKey();
        seconds10 = 0;
             
        switch(key)
        {
            case 6: 
                if(game_mode > 1)
                    game_mode--; 
                _delay_ms(100);
                 break;
            case 5:
                pgm_read_block( &currentGame[0], &Games[game_mode-1], sizeof( Games[game_mode-1] ) );
                pgm_read_block( &currentGame[1], &Games[game_mode-1], sizeof( Games[game_mode-1] ) );
                WaitForKeyUp();
                if(currentGame[0].options & GAME_OPTION_MANUAL)
                {
                    SetManual();       
                }
                pause = 1;
                if(!(currentGame[0].options & GAME_OPTION_MANUAL))
                {
                    PlayManual();
                }
                SetAll(0);
                break;
            case 4:
                if(game_mode < MODI_COUNT)
                    game_mode++;
                _delay_ms(100);
                break;
        };
        
    }
}

const uint16_t valency[] = {36000, 3600, 600, 60,10,1};
const uint16_t valency10[] = {1000,100,10,1};

void SetManual(void)
{
    //!!!!ACHTUNG!!!! !!!!Die Maske wird immer gespiegelt angegeben!!!! !!!!ACHTUNG!!!!
    uint8_t finished = 1;
    //First clear Display
    SetAll(0);
    //Start first Input for HOUR_MIN, We use all digits, so the mask is 0xFF
    finished &= KeyRo(0xFF,MODE_HOUR_MIN);
    if(!finished) return;
    //We need seconds, too. But Seconds need only 00110011 as mask
    finished &= KeyRo(0b11001100,MODE_SEC);
    if(!finished) return;
    //Jetzt setzen wir den Bonus
    if(currentGame[0].options & GAME_OPTION_BONUS) //Mchten wir berhaupt einen Bonue?
        finished &= KeyRo(0b00001111, MODE_BONUS); //Hier schon
    if(!finished) return;
    //Guillotinen setzten lassen.
    if(currentGame[0].options & GAME_OPTION_FREE_GUILLOTINE) //Der Spieler darf entscheiden wie viel Guillotinen er mchte
        finished &= KeyRo(1, MODE_GUILLOTINE_COUNT);
    if(!finished) return;
    for(uint8_t i = 0; i < ((currentGame[0].options >> 5) & 7); i++)
    {
        if(currentGame[0].options & GAME_OPTION_USE_MOVES) //Nach wieviel Zgen soll die Uhr automatisch umschalten?
            finished &= KeyRo(0b00001111, MODE_MOVES_COUNT1 + i);
        finished &= KeyRo(0b00001111, MODE_GUILLOTINE1 + i);
        if(!finished) return;
    }
    if(finished)
    {
        //Say the game we are finished
        currentGame[0].options &= ~GAME_OPTION_MANUAL;
        currentGame[1].options &= ~GAME_OPTION_MANUAL;
    } 
    SetBlackDigit(9);
}

uint8_t KeyRo(uint8_t mask, uint8_t mode)
{
    uint8_t digit = 255;
    while(!((uint8_t)(1<<++digit) & mask))
        if(digit > 6)
            return ABORT;
    while(1)
    {
        uint8_t key = 255;
        while(key == 255)
            key = GetKey();
        switch(key)
        {
            case 2: 
                if(digit == 0)
                    digit = 8;
                while(!((uint8_t)(1<<--digit) & mask))
                    if(digit < 1)
                        digit = 8;
                _delay_ms(100);
                seconds10 = 0;
                break;
            case 4:
            case 6:
                int8_t pre = key - 5;
                switch(mode)
                {
                    case MODE_HOUR_MIN:
                         Add(&currentGame[digit / 4].time, valency[digit % 4] * pre, MAX_TIME);
                         break;
                    case MODE_SEC:
                         Add(&currentGame[digit / 4].time, valency[digit % 4] * pre, MAX_TIME);
                         break;
                    case MODE_GUILLOTINE1:
                    case MODE_GUILLOTINE2:
                    case MODE_GUILLOTINE3:
                    case MODE_GUILLOTINE4:
                    case MODE_GUILLOTINE5:
                    case MODE_GUILLOTINE6:
                    case MODE_GUILLOTINE7:
                        for(uint8_t i = 0; i < 2; i++)
                            Add(&currentGame[i].guillotine[mode - MODE_GUILLOTINE1], valency[digit % 4] * pre, MAX_GUILLOTINE);
                        break;
                    case MODE_BONUS:
                        for(uint8_t i = 0; i < 2; i++)
                            Add(&currentGame[i].bonus, valency[digit % 4] * pre, MAX_BONUS);
                    case MODE_GUILLOTINE_COUNT:
                        for(uint8_t i = 0; i < 2; i++)
                            Add(&currentGame[current].options, (1 << 5), 224);
                        break;
                    case MODE_MOVES_COUNT1:
                    case MODE_MOVES_COUNT2:
                    case MODE_MOVES_COUNT3:
                    case MODE_MOVES_COUNT4:
                    case MODE_MOVES_COUNT5:
                    case MODE_MOVES_COUNT6:
                    case MODE_MOVES_COUNT7:   
                        for(uint8_t i = 0; i < 2; i++)
                            Add(&currentGame[current].moves[mode - MODE_MOVES_COUNT1], valency10[digit % 4] * pre, 7);
                        break;
                    case MODE_MOVES:
                        Add(&moves, valency10[digit % 4] * pre, MAX_MOVES);
                        break;
                    _delay_ms(50);
                };
                break;
            /*case 4:
                switch(mode)
                {
                    case MODE_HOUR_MIN:
                        currentGame[digit / 4].time += valency[digit % 4];
                        break;
                    case MODE_SEC:
                        currentGame[digit / 4].time += valency[digit % 4 + 2];
                        break;
                    case MODE_GUILLOTINE1:
                    case MODE_GUILLOTINE2:
                    case MODE_GUILLOTINE3:
                    case MODE_GUILLOTINE4:
                    case MODE_GUILLOTINE5:
                    case MODE_GUILLOTINE6:
                    case MODE_GUILLOTINE7:
                        currentGame[digit / 4].guillotine[mode - MODE_GUILLOTINE1] += valency[digit % 4];
                        currentGame[1 - digit / 4].guillotine[mode - MODE_GUILLOTINE1] = currentGame[digit / 4].guillotine[mode - MODE_GUILLOTINE1];
                        break;   
                    case MODE_BONUS:
                        for(uint8_t i = 0; i!=2; i++)
                            currentGame[i].bonus += valency[digit % 4 + 2];
                        break;                     
                    case MODE_GUILLOTINE_COUNT:
                        for(uint8_t i = 0; i!=2; i++)
                            if(((currentGame[current].options >> 5) & 7) < 7)
                                currentGame[current].options += (1 << 5);
                        break;
                    case MODE_MOVES_COUNT1:
                    case MODE_MOVES_COUNT2:
                    case MODE_MOVES_COUNT3:
                    case MODE_MOVES_COUNT4:
                    case MODE_MOVES_COUNT5:
                    case MODE_MOVES_COUNT6:
                    case MODE_MOVES_COUNT7:
                        for(uint8_t i = 0; i!=2; i++)
                        break;                        
                };
                _delay_ms(100);
                break;*/
            case 5:
                WaitForKeyUp();
                return FINISHED;
                break;
            /*case 6:
                switch(mode)
                {
                    case MODE_HOUR_MIN:
                        if(valency[digit % 4] <= currentGame[digit / 4].time)
                            currentGame[digit / 4].time -= valency[digit % 4];
                        break;
                    case MODE_SEC:
                        if(valency[digit % 4 + 2] <= currentGame[digit / 4].time)
                            currentGame[digit / 4].time -= valency[digit % 4 + 2];
                        break;
                    case MODE_GUILLOTINE1:
                    case MODE_GUILLOTINE2:
                    case MODE_GUILLOTINE3:
                    case MODE_GUILLOTINE4:
                    case MODE_GUILLOTINE5:
                    case MODE_GUILLOTINE6:
                    case MODE_GUILLOTINE7:
                        if(valency[digit % 4] <= currentGame[digit / 4].guillotine[mode - MODE_GUILLOTINE1])
                            currentGame[digit / 4].guillotine[mode - MODE_GUILLOTINE1] -= valency[digit % 4];
                        currentGame[1 - digit / 4].guillotine[mode - MODE_GUILLOTINE1] = currentGame[digit / 4].guillotine[mode - MODE_GUILLOTINE1];
                        break;
                    case MODE_BONUS:
                        for(uint8_t i = 0; i!=2; i++)
                            if(valency[digit % 4 + 2] <= currentGame[0].bonus)
                                currentGame[i].bonus -= valency[digit % 4 + 2];
                        break;        
                    case MODE_GUILLOTINE_COUNT:
                        for(uint8_t i = 0; i!=2; i++)
                            if(((currentGame[current].options >> 5) & 7) > 0)
                                currentGame[current].options -= (1 << 5);
                        break;          
                    case MODE_MOVES:
                        
                        break;
                    case MODE_MOVES_COUNT1:
                    case MODE_MOVES_COUNT2:
                    case MODE_MOVES_COUNT3:
                    case MODE_MOVES_COUNT4:
                    case MODE_MOVES_COUNT5:
                    case MODE_MOVES_COUNT6:
                    case MODE_MOVES_COUNT7:
                        break;
                };
                _delay_ms(100);
                break;*/
            case 8:
            if(digit > 6)
                digit = 255;
                while(!((uint8_t)(1<<++digit) & mask))
                    if(digit > 6)
                        digit = 255;
                _delay_ms(100);
                seconds10 = 0;
                break;
            case 9:
                return ABORT;
                break;
        };
        //Show on Display, but first set the time to show
        SetTime();
        //Now we can call Show to show the time
        Show(digit, mode);
    }
}

void Show(uint8_t digit, uint8_t mode)
{
    //Now We show the time for each player
    uint16_t min;
    for(uint8_t i = 0; i<2;i++)
    {
        switch(mode)
        {
            case MODE_HOUR_MIN:
                Show2Digits(minutes[i],2 + 4*i);
                Show2DigitsDots(hours[i],0 + 4*i,2);
                break;
            case MODE_SEC:
                SetDigit(0 + 4*i,0);
                SetDigit(1 + 4*i,128);
                Show2Digits(seconds[i],2 + 4*i);
                break;
            case MODE_GUILLOTINE1:
            case MODE_GUILLOTINE2:
            case MODE_GUILLOTINE3:
            case MODE_GUILLOTINE4:
            case MODE_GUILLOTINE5:
            case MODE_GUILLOTINE6:
            case MODE_GUILLOTINE7:
                    min = currentGame[0].guillotine[mode - MODE_GUILLOTINE1] / 60;
                    Show2Digits(min % 60, 2 + 4);
                    Show2DigitsDots(min / 60, 0 + 4,2);
                    for(uint8_t o = 0; o < 3; o++)
                        SetDigit(4 + o,0);
                    SetDigit(7,chars[mode - MODE_GUILLOTINE1 + 1]);   
                break;
            case MODE_BONUS:
                    min = currentGame[0].bonus;
                    Show2Digits(min % 60, 2 + 4);
                    Show2DigitsDots(min / 60, 0 + 4,2);
                    SetDigit(4,0); 
                    SetDigit(5,31);  
                    SetDigit(6,21);
                    SetDigit(7,91);        
                break;
            case MODE_GUILLOTINE_COUNT:
                for(uint8_t o = 0; o < 7; o++)
                    SetDigit(o,0);
                SetDigit(7,chars[(currentGame[current].options >> 5) & 7]);
                break;
            case MODE_MOVES_COUNT1:
            case MODE_MOVES_COUNT2:
            case MODE_MOVES_COUNT3:
            case MODE_MOVES_COUNT4:
            case MODE_MOVES_COUNT5:
            case MODE_MOVES_COUNT6:
            case MODE_MOVES_COUNT7:
                    Show2Digits(currentGame[current].moves[mode-MODE_MOVES_COUNT1] % 100, 2 + 4);
                    Show2Digits(currentGame[current].moves[mode-MODE_MOVES_COUNT1] / 100, 0 + 4);
                    for(uint8_t o = 0; o < 3; o++)
                        SetDigit(4 + o,0);
                    SetDigit(7,chars[mode - MODE_MOVES_COUNT1 + 1]);   
                break;
            case MODE_MOVES:
                    ShowMoves();
                    SetDigit(4, 109);
                    SetDigit(5, 62);
                    SetDigit(6, 94);
                    SetDigit(7, 0);
                break;
        };
    }
    if(seconds10 < 5 )
    {
        SetDigit(digit,0);
        SetBlackDigit(digit);
    }
    else
        SetBlackDigit(9);
}

void Add(uint32_t* source, int32_t add, uint32_t max)
{
    int64_t tmp = (int64_t)*source + add;
    if(tmp > 0 && tmp <= max)
        *source = tmp;
}

void pgm_read_block( void * dest, void * source, size_t byteCnt )
{
  uint8_t* destPtr = (uint8_t *)dest;
  uint8_t* srcPtr  = (uint8_t *)source;
  size_t i;

  for(i = 0; i < byteCnt; ++i )
    *destPtr++ = pgm_read_byte( srcPtr++ );
}

