#include <avr/io.h>
#include <avr/interrupt.h>
#include <avr/pgmspace.h>
#ifndef F_CPU
/* Definiere F_CPU, wenn F_CPU nicht bereits vorher definiert
(z.B. durch bergabe als Parameter zum Compiler innerhalb
des Makefiles). Zustzlich Ausgabe einer Warnung, die auf die
"nachtrgliche" Definition hinweist */
#warning "F_CPU war noch nicht definiert, wird nun mit 8000000 definiert"
#define F_CPU 7998976 /* Quarz mit 8 Mhz */
#endif 
#include <util/delay.h>
#include "Max7219.h"
#include "utils.h"

#ifndef CHESS
#define CHESS

#define GAME_OPTION_MANUAL 1
#define GAME_OPTION_HOURGLASS 2
#define GAME_OPTION_BONUS 4
#define GAME_OPTION_LATE_BONUS 8
#define GAME_OPTION_FREE_BONUS 16

#define GAME_OPTION_GUILLOTINE1 32
#define GAME_OPTION_GUILLOTINE2 64
#define GAME_OPTION_GUILLOTINE4 128

#define GAME_OPTION_FREE_GUILLOTINE 256
#define GAME_OPTION_WDH_GUILLOTINE 512
#define GAME_OPTION_BYOYOMI 1024
#define GAME_OPTION_KBYOYOMI 2048
#define GAME_OPTION_USE_MOVES 4096

typedef struct 
{
    uint32_t time;
    uint16_t bonus;
    uint16_t options;
    uint16_t guillotine[7];
    uint8_t moves[7];
} Game;

Game create_game(uint16_t time, uint16_t bonus);

void ChessInit(void);
void PlayManual(void);
void SetManual(void);
void SetTime(void);
void SetTimeForPlayer(uint8_t player, uint32_t time);
void Lost(uint8_t player);
void MainMenu(void);
void pgm_read_block( void * dest, void * source, size_t byteCnt );
uint8_t KeyRo(uint8_t mask, uint8_t mode);
void Show(uint8_t digit, uint8_t mode);
void DoBonus(uint8_t player);
void DoBonusFree(uint8_t player, uint32_t);
void AddTime(uint8_t player, uint16_t time);
uint32_t GetTime(uint8_t player);
void DoMoves(uint8_t player);
void DoGuillotine(void);
void ShowMoves(void);
void Add(uint32_t* source, int32_t add, uint32_t max);

#endif
